import com.dukascopy.api.*;
import java.text.SimpleDateFormat;
import java.util.*;

@RequiresFullAccess
public class simplebearishkinkfinderinputstarttimeandxv3 implements IStrategy {
    private IContext context;
    private IConsole console;
    private IHistory history;

    // Parameters
    @Configurable("X Price")
    public double xPrice = 1.32351;

    @Configurable("Start Time (yyyy-MM-dd HH:mm:ss)")
    public String humanReadableStartTime = "2022-01-01 00:00:00";

    private long startTime;
    private int kinkCount = 0;
    private List<Kink> kinks = new ArrayList<>();

    @Override
    public void onStart(IContext context) throws JFException {
        this.context = context;
        this.console = context.getConsole();
        this.history = context.getHistory();

        try {
            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            dateFormat.setTimeZone(TimeZone.getTimeZone("UTC")); // Set UTC timezone for consistency
            startTime = dateFormat.parse(humanReadableStartTime).getTime();
        } catch (Exception e) {
            console.getErr().println("Invalid date format for Start Time. Use 'yyyy-MM-dd HH:mm:ss'.");
            return;
        }

        if (xPrice == 0) {
            console.getOut().println("X price not set. Please configure the X price.");
            return;
        }

        console.getOut().println("Strategy started. Searching for Bearish KINKs...");
        searchForBearishKinks();

        if (!kinks.isEmpty()) {
            console.getOut().println("Kinks detected. Processing complete.");
        } else {
            console.getOut().println("No kinks found.");
        }
    }

    private void searchForBearishKinks() throws JFException {
        long currentTime = startTime;
        long timeStep = 1000 * 60 * 60; // 1-hour chunks for historical data

        console.getOut().println("Searching for kinks...");
        List<Kink> detectedKinks = new ArrayList<>();

        while (true) {
            List<ITick> ticks = history.getTicks(Instrument.GBPUSD, currentTime - timeStep, currentTime);
            if (ticks.isEmpty()) {
                console.getOut().println("No more historical data available.");
                break;
            }

            // Process ticks from left to right
            boolean crossedX = false;
            boolean waitingForMinPriceTouch = false;
            double tempMinPrice = Double.MAX_VALUE; // Temporary minimum price
            double minPrice = Double.MAX_VALUE;     // Confirmed minimum price

            for (int i = 1; i < ticks.size(); i++) {
                double currentBid = ticks.get(i).getBid();
                double prevBid = ticks.get(i - 1).getBid();

                // Step 1: Detect crossing X downward
                if (!crossedX && prevBid > xPrice && currentBid <= xPrice) {
                    crossedX = true;
                    tempMinPrice = currentBid; // Start tracking minimum price
                }

                // Step 2: Track minimum price after crossing
                if (crossedX && currentBid < tempMinPrice) {
                    tempMinPrice = currentBid; // Update to the lowest price so far
                }

                // Step 3: Confirm minPrice once price returns to X
                if (crossedX && currentBid == xPrice && prevBid < xPrice) {
                    minPrice = tempMinPrice; // Finalize the minimum price
                    waitingForMinPriceTouch = true;
                }

                // Step 4: Detect kink formation or new lower minPrice
                if (waitingForMinPriceTouch && currentBid <= minPrice) {
                    // Kink is formed if the price touches or crosses minPrice
                    if (xPrice - minPrice > 0.000015) { // Adjusted tolerance
                        Kink newKink = new Kink(ticks.get(i).getTime(), xPrice, minPrice);
                        detectedKinks.add(newKink);

                        // Allow multiple kinks within the chunk, no limit
                    }

                    // Reset for the next potential kink but allow for new lower minPrice
                    waitingForMinPriceTouch = false;
                    crossedX = true; // Continue tracking for new minPrice
                    tempMinPrice = currentBid; // Start tracking a new minPrice
                }

                // Reset minPrice when the price crosses back above X
                if (crossedX && currentBid > xPrice) {
                    crossedX = false; // Reset crossing state
                    tempMinPrice = Double.MAX_VALUE; // Reset tempMinPrice
                    minPrice = Double.MAX_VALUE; // Reset confirmed minPrice
                }
            }

            currentTime -= timeStep;
        }

        // Sort detected kinks from closest to start time backward
        detectedKinks.sort((k1, k2) -> Long.compare(k2.startTime, k1.startTime));
        kinks = detectedKinks;
        kinkCount = kinks.size();

        for (int i = 0; i < kinks.size(); i++) {
            console.getOut().println("KINK " + (i + 1) + ": " + kinks.get(i));
        }
    }

    // Inner Kink class
    private static class Kink {
        long startTime;
        double xPrice;
        double minPrice;

        Kink(long startTime, double xPrice, double minPrice) {
            this.startTime = startTime;
            this.xPrice = xPrice;
            this.minPrice = minPrice;
        }

        @Override
        public String toString() {
            return "Kink{" +
                    "startTime=" + new Date(startTime) +
                    ", xPrice=" + xPrice +
                    ", minPrice=" + minPrice +
                    '}';
        }
    }

    @Override
    public void onTick(Instrument instrument, ITick tick) throws JFException {
        // Not used
    }

    @Override
    public void onBar(Instrument instrument, Period period, IBar askBar, IBar bidBar) throws JFException {
        // Not used
    }

    @Override
    public void onMessage(IMessage message) throws JFException {
        // Not used
    }

    @Override
    public void onAccount(IAccount account) throws JFException {
        // Not used
    }

    @Override
    public void onStop() throws JFException {
        console.getOut().println("Strategy stopped.");
    }
}
